var finesse = finesse || {};
finesse.gadget = finesse.gadget || {};
finesse.container = finesse.container || {};
clientLogs = finesse.cslogger.ClientLogger || {};  // for logging

// Gadget Config needed for instantiating ClientServices
/** @namespace */
finesse.gadget.Config = (function () {
  var _prefs = new gadgets.Prefs();

  /** @scope finesse.gadget.Config */
  return {
    authorization: _prefs.getString("authorization"),
    country: _prefs.getString("country"),
    language: _prefs.getString("language"),
    locale: _prefs.getString("locale"),
    host: _prefs.getString("host"),
    hostPort: _prefs.getString("hostPort"),
    extension: _prefs.getString("extension"),
    mobileAgentMode: _prefs.getString("mobileAgentMode"),
    mobileAgentDialNumber: _prefs.getString("mobileAgentDialNumber"),
    xmppDomain: _prefs.getString("xmppDomain"),
    pubsubDomain: _prefs.getString("pubsubDomain"),
    restHost: _prefs.getString("restHost"),
    scheme: _prefs.getString("scheme"),
    localhostFQDN: _prefs.getString("localhostFQDN"),
    localhostPort: _prefs.getString("localhostPort"),
    clientDriftInMillis: _prefs.getInt("clientDriftInMillis")
  };
}());

/** @namespace */
finesse.modules = finesse.modules || {};
finesse.modules.JabraDeviceIntegrationGadget = (function($) {
  var user,
    states,
    dialogs,
    dlg,
    wsocket,

    /**
     * Populates the fields in the gadget with data
     */
    render = function() {
      // Not used
      var currentState = user.getState();
      gadgets.window.adjustHeight();
    },

    // Handler to handle changes in a dialog
    _processCall = function(dialog) {
      // Not used
    },

    /**
     *  Handler for additions to the Dialogs collection object.  This will occur when a new
     *  Dialog is created on the Finesse server for this user.
     */
    handleNewDialog = function(dialog) {
      // Not used
      dlg = dialog;
      // add a change handler to the dialog
      dialog.addHandler('change', _processCall);
    },

    /**
     *  Handler for deletions from the Dialogs collection object for this user.  This will occur
     *  when a Dialog is removed from this user's collection (example, end call)
     */
    handleEndDialog = function(dialog) {
      // Not used
    },

    /**
     * Handler for the onLoad of a User object.  This occurs when the User object is initially read
     * from the Finesse server.  Any once only initialization should be done within this function.
     */
    handleUserLoad = function(userevent) {
      // Get an instance of the dialogs collection and register handlers for dialog additions and
      // removals
      dialogs = user.getDialogs({
        onCollectionAdd: handleNewDialog,
        onCollectionDelete: handleEndDialog
      });
      render();
    },

    /**
     *  Handler for all User updates
     */
    handleUserChange = function(userevent) {
      render();
    };


  /** @scope finesse.modules.JabraDeviceIntegrationGadget */
  return {

    /**
     * Record call
     */
    recordCall: function() {
      dlg.requestAction(user.getExtension(),
        finesse.restservices.Dialog.Actions.START_RECORDING,
        {
          success: function(rsp) {},
          error: function(rsp) {}
        });
    },

    /**
     * Drop call
     */
    dropCall: function() {
      dlg.requestAction(user.getExtension(),
        finesse.restservices.Dialog.Actions.DROP,
        {
          success: function(rsp) {},
          error: function(rsp) {}
        });
    },

    /**
     * Logout
     */
    logoutUser: function() {
      user.logout();
    },

    /**
     * toggleStatus
     */
    toggleStatus: function() {
      if (user._data.state === states.NOT_READY) {
        // Not ready -> Ready
        user.getNotReadyReasonCodes(
        {
          success: function(reasonCodes) {
            if (reasonCodes === undefined) {
              user.setState(states.READY);
            } else {
              // Find the next code id
              var currentNotReadyCodeId = parseInt(user.getNotReadyReasonCodeId());
              var nextReason;
              var index;
              for (index = 0; index < reasonCodes.length; ++index) {
                if (parseInt(reasonCodes[index].id) > currentNotReadyCodeId) {
                  if (nextReason === undefined) {
                    nextReason = reasonCodes[index];
                  } else if ((parseInt(reasonCodes[index].id) - currentNotReadyCodeId) <
                  (parseInt(nextReason.id) - currentNotReadyCodeId)) {
                    nextReason = reasonCodes[index];
                  }
                }
              }
              if (nextReason === undefined) {
                user.setState(states.READY);
              } else {
                user.setState(states.NOT_READY, nextReason);
              }
            }
          },
          error: function(rsp) {
            clientLogs.log("getNotReadyReasonCodes error");
          }
        });
      } else if (user._data.state === states.READY) {
        // Ready -> first not-ready
        user.getNotReadyReasonCodes(
        {
          success: function(reasonCodes) {
            if (reasonCodes === undefined) {
              // No reason codes
              // Now change to not-ready
              user.setState(states.NOT_READY);
            } else {
              // Found reason codes
              var reason = reasonCodes[0];
              var index;
              for (index = 1; index < reasonCodes.length; ++index) {
                if (parseInt(reasonCodes[index].id) < reason.id) {
                  reason = reasonCodes[index];
                }
              }
              // Now change to not-ready
              user.setState(states.NOT_READY, reason);
            }
          },
          error: function(rsp) {
            clientLogs.log("getNotReadyReasonCodes error");
          }
        });
      } else if (user._data.state === states.TALKING) {
        // Talking -> first not-ready
        user.getNotReadyReasonCodes(
        {
          success: function(reasonCodes) {
            if (reasonCodes === undefined) {
              // Now change to not-ready
              user.setState(states.NOT_READY, reason);
            } else {
              var reason = reasonCodes[0];
              var index;
              for (index = 1; index < reasonCodes.length; ++index) {
                if (parseInt(reasonCodes[index].id) < reason.id) {
                  reason = reasonCodes[index];
                }
              }
              // Now change to not-ready
              user.setState(states.NOT_READY, reason);
            }
          },
          error: function(rsp) {
            clientLogs.log("getNotReadyReasonCodes error");
          }
        });
      }
    },

    /**
     * Performs all initialization for this gadget
     */
    init: function() {
      var prefs = new gadgets.Prefs(),
        id = prefs.getString("id");
      var clientLogs = finesse.cslogger.ClientLogger; // declare clientLogs

      gadgets.window.adjustHeight();

      // Initiate the ClientServices and load the user object.  ClientServices are
      // initialized with a reference to the current configuration.
      finesse.clientservices.ClientServices.init(finesse.gadget.Config);
      clientLogs.init(gadgets
        .Hub,
        "JabraDeviceIntegrationGadget"); //this gadget id will be logged as a part of the message
      user = new finesse.restservices.User({
        id: id,
        onLoad: handleUserLoad,
        onChange: handleUserChange
      });

      states = finesse.restservices.User.States;
      clientLogs.log("completed init in JabraDeviceIntegrationGadget");
    },

    /**
     * connectWebsocket
     */
    connectWebsocket: function() {
      $("#status").text("Trying to connect to Jabra CC Agent App...");

      wsocket = new WebSocket("wss://localjabrawebsocketserver:8080");

      wsocket.onmessage = function(evt) {
        if (evt.data.indexOf("DeviceAttached;") > -1) {
          var values = evt.data.split(';');
          var noOfDevices = values[1];
          var deviceName = values[2];
          if (noOfDevices === "0") {
            $("#connecteddevice").text("Connected device: -");
          } else {
            $("#connecteddevice").text("Connected device: " + deviceName);
          }
        }
        if (evt.data.indexOf("CustomButton1;True") > -1) {
          //finesse.modules.JabraDeviceIntegrationGadget.logoutUser();
          finesse.modules.JabraDeviceIntegrationGadget.recordCall();
          //finesse.modules.JabraDeviceIntegrationGadget.toggleStatus();
        }
        if (evt.data.indexOf("CustomButton2;True") > -1) {
          //finesse.modules.JabraDeviceIntegrationGadget.logoutUser();
          //finesse.modules.JabraDeviceIntegrationGadget.recordCall();
          finesse.modules.JabraDeviceIntegrationGadget.toggleStatus();
        }
      }

      wsocket.onopen = function() {
        $("#status").text("Connected to Jabra CC Agent App");
        wsocket.send("GetDevices");
      }

      wsocket.onclose = function() {
        $("#status").text("Unable to connect to Jabra CC Agent App");
        setTimeout(function() { finesse.modules.JabraDeviceIntegrationGadget.connectWebsocket(); }, 5000); // retry in 5 seconds
      }
    }

  };
}(jQuery));